@@ -70,8 +70,8 @@ module Text.PrettyPrint.Leijen.Text (
    Doc,
 
    -- * Basic combinators
-   empty, char, text, (<>), nest, line, linebreak, group, softline,
-   softbreak, spacebreak,
+   empty, isEmpty, char, text, textStrict, (<>), nest, line, linebreak, group,
+   softline, softbreak, spacebreak,
 
    -- * Alignment
    --
@@ -104,7 +104,7 @@ module Text.PrettyPrint.Leijen.Text (
    squote, dquote, semi, colon, comma, space, dot, backslash, equals,
 
    -- * Primitive type documents
-   string, int, integer, float, double, rational, bool,
+   string, stringStrict, int, integer, float, double, rational, bool,
 
    -- * Position-based combinators
    column, nesting, width,
@@ -114,28 +114,34 @@ module Text.PrettyPrint.Leijen.Text (
 
    -- * Rendering
    SimpleDoc(..), renderPretty, renderCompact, renderOneLine,
-   displayB, displayT, displayIO, putDoc, hPutDoc
+   displayB, displayT, displayTStrict, displayIO, putDoc, hPutDoc
 
    ) where
 
-#if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 710
-import Prelude hiding ((<$>))
-#endif
+import Prelude        ()
+import Prelude.Compat hiding ((<$>))
 
-import Data.String (IsString (..))
+import Data.String (IsString(..))
 import System.IO   (Handle, hPutChar, stdout)
 
 import           Data.Int               (Int64)
-import           Data.Monoid            (Monoid (..))
+import           Data.List              (intersperse)
+import           Data.Monoid            (Monoid(..))
+import qualified Data.Text              as TS
 import           Data.Text.Lazy         (Text)
 import qualified Data.Text.Lazy         as T
 import           Data.Text.Lazy.Builder (Builder)
 import qualified Data.Text.Lazy.Builder as B
 import qualified Data.Text.Lazy.IO      as T
 
+#if MIN_VERSION_base (4,9,0)
+import Data.Semigroup (Semigroup(..))
+#else
+import Data.Monoid ((<>))
+#endif
 
 infixr 5 </>,<//>,<$>,<$$>
-infixr 6 <>,<+>,<++>
+infixr 6 <+>,<++>
 
 
 -----------------------------------------------------------
@@ -227,8 +233,8 @@ encloseSep left right sp ds
 --   (If you want put the commas in front of their elements instead of
 --   at the end, you should use 'tupled' or, in general, 'encloseSep'.)
 punctuate :: Doc -> [Doc] -> [Doc]
-punctuate _ [] = []
-punctuate _ [d] = [d]
+punctuate _ []     = []
+punctuate _ [d]    = [d]
 punctuate p (d:ds) = (d <> p) : punctuate p ds
 
 
@@ -324,12 +330,6 @@ fold      :: (Doc -> Doc -> Doc) -> [Doc] -> Doc
 fold _ [] = empty
 fold f ds = foldr1 f ds
 
--- | The document @(x \<\> y)@ concatenates document @x@ and document
---   @y@. It is an associative operation having 'empty' as a left and
---   right unit.  (infixr 6)
-(<>)   :: Doc -> Doc -> Doc
-x <> y = x `beside` y
-
 -- | The document @(x \<+\> y)@ concatenates document @x@ and @y@ with
 --   a 'space' in between.  (infixr 6)
 (<+>) :: Doc -> Doc -> Doc
@@ -516,41 +516,40 @@ equals = char '='
 --   characters. It is used instead of 'text' whenever the text
 --   contains newline characters.
 string :: Text -> Doc
-string str = case T.uncons str of
-               Nothing          -> empty
-               Just ('\n',str') -> line <> string str'
-               _                -> case (T.span (/='\n') str) of
-                                     (xs,ys) -> text xs <> string ys
+string = mconcat . intersperse line . map text . T.lines
+
+stringStrict :: TS.Text -> Doc
+stringStrict = mconcat . intersperse line . map textStrict . TS.lines
 
 -- | The document @(bool b)@ shows the literal boolean @b@ using
 --   'text'.
 bool   :: Bool -> Doc
-bool b = text' b
+bool = text'
 
 -- | The document @(int i)@ shows the literal integer @i@ using
 --   'text'.
 int   :: Int -> Doc
-int i = text' i
+int = text'
 
 -- | The document @(integer i)@ shows the literal integer @i@ using
 --   'text'.
 integer   :: Integer -> Doc
-integer i = text' i
+integer = text'
 
 -- | The document @(float f)@ shows the literal float @f@ using
 --   'text'.
 float   :: Float -> Doc
-float f = text' f
+float = text'
 
 -- | The document @(double d)@ shows the literal double @d@ using
 --   'text'.
 double   :: Double -> Doc
-double d = text' d
+double = text'
 
 -- | The document @(rational r)@ shows the literal rational @r@ using
 --   'text'.
 rational   :: Rational -> Doc
-rational r = text' r
+rational = text'
 
 text' :: (Show a) => a -> Doc
 text' = text . T.pack . show
@@ -577,40 +576,43 @@ instance Pretty Doc where
 instance Pretty Text where
   pretty = string
 
+instance Pretty TS.Text where
+  pretty = stringStrict
+
 instance Pretty () where
   pretty () = text' ()
 
 instance Pretty Bool where
-  pretty b = bool b
+  pretty = bool
 
 instance Pretty Char where
-  pretty c = char c
+  pretty = char
 
-  prettyList s = string $ T.pack s
+  prettyList = string . T.pack
 
 instance Pretty Int where
-  pretty i = int i
+  pretty = int
 
 instance Pretty Integer where
-  pretty i = integer i
+  pretty = integer
 
 instance Pretty Float where
-  pretty f = float f
+  pretty = float
 
 instance Pretty Double where
-  pretty d = double d
+  pretty = double
 
 --instance Pretty Rational where
 --  pretty r = rational r
 
-instance (Pretty a,Pretty b) => Pretty (a,b) where
+instance (Pretty a, Pretty b) => Pretty (a,b) where
   pretty (x,y) = tupled [pretty x, pretty y]
 
-instance (Pretty a,Pretty b,Pretty c) => Pretty (a,b,c) where
+instance (Pretty a, Pretty b, Pretty c) => Pretty (a,b,c) where
   pretty (x,y,z)= tupled [pretty x, pretty y, pretty z]
 
 instance Pretty a => Pretty (Maybe a) where
-  pretty Nothing = empty
+  pretty Nothing  = empty
 
   pretty (Just x) = pretty x
 
@@ -638,7 +640,7 @@ instance Pretty a => Pretty (Maybe a) where
 --   @
 fillBreak     :: Int -> Doc -> Doc
 fillBreak f x = width x (\w ->
-                          if (w > f)
+                          if w > f
                             then nest f linebreak
                             else spaced (f - w)
                         )
@@ -668,7 +670,7 @@ fillBreak f x = width x (\w ->
 --   @
 fill     :: Int -> Doc -> Doc
 fill f d = width d (\w ->
-                     if (w >= f)
+                     if w >= f
                        then empty
                        else spaced (f - w)
                    )
@@ -773,6 +775,14 @@ data Doc = Empty
 instance IsString Doc where
   fromString = string . T.pack
 
+-- | In particular, note that the document @(x '<>' y)@ concatenates
+--   document @x@ and document @y@. It is an associative operation
+--   having 'empty' as a left and right unit.  (infixr 6)
+#if MIN_VERSION_base (4,9,0)
+instance Semigroup Doc where
+    (<>) = beside
+
+#endif
 instance Monoid Doc where
     mempty  = empty
     mappend = beside
@@ -796,6 +806,11 @@ data SimpleDoc = SEmpty
 empty :: Doc
 empty = Empty
 
+-- | Determine if the document is empty or not.
+isEmpty :: Doc -> Bool
+isEmpty Empty = True
+isEmpty _     = False
+
 -- | The document @(char c)@ contains the literal character @c@. The
 --   character shouldn't be a newline (@'\n'@), the function 'line'
 --   should be used for line breaks.
@@ -812,6 +827,11 @@ text s
   | T.null s  = Empty
   | otherwise = Text (T.length s) (B.fromLazyText s)
 
+textStrict :: TS.Text -> Doc
+textStrict s
+  | TS.null s  = Empty
+  | otherwise  = Text (fromIntegral $ TS.length s) (B.fromText s)
+
 -- | The @line@ document advances to the next line and indents to the
 --   current nesting level. Document @line@ behaves like @(text \"
 --   \")@ if the line break is undone by 'group' or if rendered with
@@ -1011,14 +1031,16 @@ c `consB` b = B.singleton c `mappend` b
 displayT :: SimpleDoc -> Text
 displayT = B.toLazyText . displayB
 
+displayTStrict :: SimpleDoc -> TS.Text
+displayTStrict = T.toStrict . displayT
+
 -- | @(displayIO handle simpleDoc)@ writes @simpleDoc@ to the
 --   file handle @handle@. This function is used for example by
 --   'hPutDoc':
 --
 --   > hPutDoc handle doc = displayIO handle (renderPretty 0.4 100 doc)
 displayIO :: Handle -> SimpleDoc -> IO ()
-displayIO handle simpleDoc
- = display simpleDoc
+displayIO handle = display
     where
       display SEmpty        = return ()
       display (SChar c x)   = hPutChar handle c >> display x
@@ -1051,7 +1073,7 @@ instance Show SimpleDoc where
 -- hello world
 -- @
 putDoc :: Doc -> IO ()
-putDoc doc = hPutDoc stdout doc
+putDoc = hPutDoc stdout
 
 -- | @(hPutDoc handle doc)@ pretty prints document @doc@ to the file
 --   handle @handle@ with a page width of 100 characters and a ribbon
